=begin
  gettext.rb - GetText module

  Copyright (C) 2001,2002  Masahiro Sakai, Masao Mutoh

      Masahiro Sakai    <s01397ms@sfc.keio.ac.jp>
      Masao Mutoh       <mutoh@highway.ne.jp>

  You may redistribute it and/or modify it under the same
  license terms as Ruby.

  $Id: gettext.rb,v 1.8 2002/02/22 14:05:54 mutoh Exp $
=end

require 'mo'
require 'rbconfig'

module GetText
  class TextDomain
    def initialize(name, path = nil, locale = nil)
      @name, @path = name, path
      @search_files = Array.new

	  if path
		@locale_dirs = [path]
	  else
		@locale_dirs = []
		prefix = Config::CONFIG["prefix"]
		if prefix != "/usr" and prefix != "/usr/local"
		  @locale_dirs += [prefix + "/share/locale", 
			               prefix + "/local/share/locale"]
		end
		@locale_dirs += ["/usr/share/locale", "/usr/local/share/locale"]
	  end
		
      @mofiles = Hash.new
      unless locale
		["LANG", "LC_ALL", "LC_MESSAGES"].each do |env|
		  break if locale = ENV[env]
		end
      end
	  locale = "C" unless locale
      setlocale(locale)

	  if $DEBUG
		print "Search path:"
        p @locale_dirs
		print "locale:"
		p locale
	  end
    end
    
    def setlocale(locale)
      @locales = []
      matched = /^([A-Za-z]+)(_[A-Za-z]+)?(\.\w+)?((?:@[^@]*)*)/.match(locale)
      if matched.size > 1
		matched = matched.to_a
		matched.shift
		while not matched.empty?
		  @locales.push(matched.join(''))
		  matched.pop
		end
		@locales.uniq!
      else
		@locales.push(locale)
      end
      load_mo
    end
    
    def gettext(msgid)
      if @mo
		result = @mo[msgid] ? @mo[msgid] : msgid
      else
		if $DEBUG
		  print("\nMO file is not found in\n")
		  @search_files.each do |fname|
			print("\t#{fname}\n")
		  end
		end
		result = msgid
      end
      result
    end
    
    def same_property?(name, path, locale)
      @name == name and @path == path and @locales.include?(locale)
    end
    
    private
    def load_mo
      @locales.each do |locale|
		if @mo = @mofiles[locale]
		  return @mo
		end
      end
      @locale_dirs.each do |dir|
		@locales.each{|locale|
		  fname = File.join(dir, locale, "LC_MESSAGES", @name + ".mo")
		  @search_files << fname
		  if File.exist?(fname)
			@mo = MOFile.open(fname)
			@mofiles[locale] = @mo
			break
		  end
		}
      end
    end
  end
  
  def bindtextdomain(domainname, path = nil, locale = nil)
    @@__textdomain = Hash.new unless defined?(@@__textdomain)
    textdomain = @@__textdomain[callersrc]
    unless textdomain and textdomain.same_property?(domainname, path, locale)
      @@__textdomain[callersrc] = TextDomain.new(domainname, path, locale)
    end
  end
  
  def gettext(msgid)
    @@__textdomain[callersrc].gettext(msgid)
  end
  
  def N_(msgid)
    msgid
  end

  def callersrc
    caller(2)[0].sub(/:\d+(?::in \`\S+\')?\Z/, '')
  end

  def setlocale(locale)
    @@__textdomain[callersrc].setlocale(locale)
  end
  
  alias :_ :gettext
  
  module_function :bindtextdomain, :_, :N_, :gettext, :setlocale, :callersrc
end
